/* misc.c. */

/* VIS-5D version 3.0 */

/*
vis5d program for visualizing five dimensional gridded data sets
Copyright (C) 1990, 1991, 1992  Bill Hibbard, Brian Paul, Dave Santek,
and Andre Battaiola.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 1, or (at your option)
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/



#include <math.h>
#include "globals.h"
#include "memory.h"
#include "sync.h"



/*** round ************************************************************
   Round off x to a 'nice' value.
**********************************************************************/
float round( x )
float x;
{
   float base, fudge;
   int temp;

   for (base=1000000.0; fabs(x/base)<1.0 && base>0.000001; base/=10.0);

   fudge = (x>0.0) ? 0.5 : -0.5;
   temp = (int) (x/base + fudge);
   return temp * base;
}



static void del_traj( t )
int t;
{
   int bytes;

   bytes = TrajTable[t].length * sizeof(int_2);
   deallocate( TrajTable[t].vx, bytes );
   deallocate( TrajTable[t].vy, bytes );
   deallocate( TrajTable[t].vz, bytes );
   if (TrajTable[t].kind==1) {
      bytes = TrajTable[t].length * sizeof(int_1);
      deallocate( TrajTable[t].nx, bytes );
      deallocate( TrajTable[t].ny, bytes );
      deallocate( TrajTable[t].nz, bytes );
   }
   bytes = NumTimes * sizeof(uint_2);
   deallocate( TrajTable[t].start, bytes );
   deallocate( TrajTable[t].len, bytes );
}



/*** del_last_traj ****************************************************
   Delete the most recent trajectory.
**********************************************************************/
void del_last_traj()
{
   LOCK_ON( Gfx_Lock );

   if (NumTraj) {
      del_traj( NumTraj-1 );
      NumTraj--;
   }

   LOCK_OFF( Gfx_Lock );
}




/*** del_traj_group ***************************************************
   Delete a group of trajectories.
   Input:  g - the trajectory group number in [0..TRAJGROUPS-1].
**********************************************************************/
void del_traj_group( g )
int g;
{
   struct traj *dest, *src;
   int i, num, bytes;

   LOCK_ON( Gfx_Lock );

   i = 0;
   while (i<NumTraj) {
      if (TrajTable[i].group==g) {
	 if (i<MAXTRAJ-1) {
	    del_traj( i );
	    /* delete this one by shifting remaining trajectories backward */
	    dest = &TrajTable[i];
	    src = &TrajTable[i+1];
	    num = NumTraj-i-1;
	    if (num>0)
	       memcpy( dest, src, num*sizeof(struct traj) );
	 }
	 NumTraj--;
      }
      else {
	 i++;
      }
   }

   LOCK_OFF( Gfx_Lock );
}



/***********************************************************************/
/** Graphics locking:  these functions are used to synchronize access **/
/** to the isosurfaces, slices, trajectories, etc.                    **/
/***********************************************************************/


/*** wait_lock ********************************************************
   Wait until a graphics lock can be locked.  
**********************************************************************/
int wait_lock( lk )
int *lk;
{
   int val;

   do {
      LOCK_ON( Gfx_Lock );
      val = *lk;
      if (val==0)
	 *lk = 1;
      LOCK_OFF( Gfx_Lock );
   } while (val==1);

}


/*** clear_lock *******************************************************
   Clear a lock on a graphic.
**********************************************************************/
int clear_lock( lk )
int *lk;
{
   LOCK_ON( Gfx_Lock );
   *lk = 0;
   LOCK_OFF( Gfx_Lock );
}



/*** cond_lock ********************************************************
   Do a conditional lock:  If the lock is already set, return 0,
   otherwise set the lock and return 1.
**********************************************************************/
int cond_lock( lk )
int *lk;
{

   int val;

   LOCK_ON( Gfx_Lock );
   if (*lk==0) {
      *lk = 1;
      val = 1;
   }
   else {
      val = 0;
   }
   LOCK_OFF( Gfx_Lock );

   return val;
}




int free_isosurface( time, parm )
int time, parm;
{
   int b1, b2, b3;

   if (SurfTable[parm][time].valid) {
      /* vertices */
      b1 = SurfTable[parm][time].numverts * sizeof(int_2);
      if (b1) {
	 deallocate( SurfTable[parm][time].vx, b1 );
	 deallocate( SurfTable[parm][time].vy, b1 );
	 deallocate( SurfTable[parm][time].vz, b1 );
      }
      /* normals */
      b2 = SurfTable[parm][time].numverts * sizeof(int_1);
      if (b2) {
	 deallocate( SurfTable[parm][time].nx, b2 );
	 deallocate( SurfTable[parm][time].ny, b2 );
	 deallocate( SurfTable[parm][time].nz, b2 );
      }
      /* indices */
      b3 = SurfTable[parm][time].numpts * sizeof(uint_2);
      if (b3) {
	 deallocate( SurfTable[parm][time].pts, b3 );
      }
      SurfTable[parm][time].valid = 0;
   }

   return b1+b2+b3;
}



int free_hslice( time, parm )
int time, parm;
{
   int b1, b2, b3;

   if (HSliceTable[parm][time].valid) {
      b1 = HSliceTable[parm][time].num1 * sizeof(int_2);
      if (b1) {
	 deallocate( HSliceTable[parm][time].vx1, b1 );
	 deallocate( HSliceTable[parm][time].vy1, b1 );
      }
      b2 = HSliceTable[parm][time].num2 * sizeof(int_2);
      if (b2) {
	 deallocate( HSliceTable[parm][time].vx2, b2 );
	 deallocate( HSliceTable[parm][time].vy2, b2 );
      }
      b3 = HSliceTable[parm][time].num3 * sizeof(int_2);
      if (b3) {
	 deallocate( HSliceTable[parm][time].vx3, b3 );
	 deallocate( HSliceTable[parm][time].vy3, b3 );
      }
      HSliceTable[parm][time].valid = 0;
   }

   return b1+b2+b3;
}




int free_vslice( time, parm )
int time, parm;
{
   int b1, b2, b3;

   if (VSliceTable[parm][time].valid) {
      if (VSliceTable[parm][time].valid && VSliceTable[parm][time].num1) {
	 b1 = VSliceTable[parm][time].num1 * sizeof(int_2);
	 if (b1) {
	    deallocate( VSliceTable[parm][time].vx1, b1 );
	    deallocate( VSliceTable[parm][time].vy1, b1 );
	    deallocate( VSliceTable[parm][time].vz1, b1 );
	 }
      }
      if (VSliceTable[parm][time].valid && VSliceTable[parm][time].num2) {
	 b2 = VSliceTable[parm][time].num2 * sizeof(int_2);
	 if (b2) {
	    deallocate( VSliceTable[parm][time].vx2, b2 );
	    deallocate( VSliceTable[parm][time].vy2, b2 );
	    deallocate( VSliceTable[parm][time].vz2, b2 );
	 }
      }
      if (VSliceTable[parm][time].valid && VSliceTable[parm][time].num3) {
	 b3 = VSliceTable[parm][time].num3 * sizeof(int_2);
	 if (b3) {
	    deallocate( VSliceTable[parm][time].vx3, b3 );
	    deallocate( VSliceTable[parm][time].vy3, b3 );
	    deallocate( VSliceTable[parm][time].vz3, b3 );
	 }
      }
      VSliceTable[parm][time].valid = 0;
   }

   return b1+b2+b3;
}



int free_chslice( time, parm )
int time, parm;
{
   int bytes;

   if (CHSliceTable[parm][time].valid) {
      bytes = Nr * Nc * sizeof(uint_1);
      deallocate( CHSliceTable[parm][time].data, bytes );
      CHSliceTable[parm][time].valid = 0;
   }

   return bytes;
}




int free_cvslice( time, parm )
int time, parm;
{
   int bytes;

   if (CVSliceTable[parm][time].valid) {
      bytes = CVSliceTable[parm][time].rows
	    * CVSliceTable[parm][time].cols * sizeof(uint_1);
      deallocate( CVSliceTable[parm][time].data, bytes );
      CVSliceTable[parm][time].valid = 0;
   }

   return bytes;
}



int free_hwind( time, ws )
int time, ws;
{
   int bytes;

   if (HWindTable[ws][time].valid) {
      bytes = HWindTable[ws][time].nvectors * 4 * sizeof(int_2);
      if (bytes) {
	 deallocate( HWindTable[ws][time].vx, bytes );
	 deallocate( HWindTable[ws][time].vy, bytes );
	 deallocate( HWindTable[ws][time].vz, bytes );
      }
      HWindTable[ws][time].valid = 0;
   }

   return bytes;
}



int free_vwind( time, ws )
int time, ws;
{
   int bytes;

   if (VWindTable[ws][time].valid) {
      bytes = VWindTable[ws][time].nvectors * 4 * sizeof(int_2);
      if (bytes) {
	 deallocate( VWindTable[ws][time].vx, bytes );
	 deallocate( VWindTable[ws][time].vy, bytes );
	 deallocate( VWindTable[ws][time].vz, bytes );
      }
      VWindTable[ws][time].valid = 0;
   }

   return bytes;
}




